;;;
;;; Operacje two dimensional arrays (matrices)
;;;

;;; Printing a two dimensional array
define printarr2(a);
  lvars a, b = boundslist(a), i, i0 = b(1), i1 = b(2),
        j, j0 = b(3), j1 = b(4);
  if length(b) /= 4 then
    mishap(a, 1, 'Two dimensional array needed')
  endif;
  printf('<%% ');
  for i from i0 to i1 do
    printf('<%% ');
    for j from j0 to j1 do
      printf(a(i, j), '%p' <> if j = j1 then '' else ', ' endif);
      if j > j0 and (j - j0) rem 10 = 0 then printf('\n') endif;
    endfor;
    printf(' %%>' <> if i = i1 then '\n' else ',\n' endif);
  endfor;
  printf(' %%>\n');
enddefine;


;;; Matix multiplication

define matmul(a, b) -> c;
  lvars ba = boundslist(a), bb = boundslist(b);
  lvars i, i0 = ba(1), i1 = ba(2);
  lvars j, j0 = bb(1), j1 = bb(2);
  lvars k, k0 = bb(3), k1 = bb(4);
  if length(ba) /= 4 then
    mishap(a, 1, 'Two dimensional array needed')
  endif;
  if length(bb) /= 4 then
    mishap(b, 1, 'Two dimensional array needed')
  endif;
  if ba(3) /= j0 or ba(4) /= j1 then 
    mishap(a, b, 2, 'Incompatible array dimensions');
  endif;
  newarray([^i0 ^i1 ^k0 ^k1], 0) -> c;
  for i from i0 to i1 do
    for k from k0 to k1 do
      for j from j0 to j1 do
        c(i, k) + a(i, j)*b(j, k) -> c(i, k);
      endfor;
    endfor;
  endfor;
enddefine;

;;; Applayin a function to all elements of a matrix
define apparr(a, f) -> na;
  lvars a, f, na, b = boundslist(a), i, i0 = b(1), i1 = b(2);
  if length(b) /= 2 then 
    mishap(a, 1, 'Two dimensional array needed')
  endif;
  newarray(b) -> na;
  for i from i0 to i1 do f(a(i)) -> na(i) endfor;
enddefine;

;;; Let 'arr' be a 2 by 2 matix (filled with 1)
lvars arr = newarray([1 2 1 2], 1);

;;; Change it to the Fibonacci matrix
0 -> arr(2, 2);

printf('arr to: ');
printarr2(arr);

;;; Compute a power 
lvars i;
for i from 1 to 5 do
  matmul(arr, arr) -> arr;
endfor;

;;; print
printf('arr to: ');
printarr2(arr);

