/*
   Miller-Rabin probabilistic primality test
*/

define mod_pow(a, k, n) -> p;
  lvars a, k, n, i, j = integer_length(k);
  1 -> p;
  for i from j by -1 to 0 do
    (p * p) mod n -> p;
    if testbit(k, i) then 
      (p * a) mod n -> p;
    endif;
  endfor;
enddefine;

;;; A single iteration of the test  
define miler_rabin1(n, k);
  lvars n, k, m, i, v;
  if gcd_n(k, n, 2) /= 1 then return(false); endif;
  n - 1 -> m;
  0 -> i ;
  while not (testbit(m, i)) do i + 1 -> i endwhile;
  m / 2**i -> m;
  mod_pow(k, m, n) -> v;
  if v = 1 or v = n - 1 then return(true) endif;
  while i > 0 and v /= n - 1 do 
    (v * v) mod n -> v ;
    i - 1 -> i;
  endwhile;
  if v = n - 1 then true else false endif;
enddefine;

;;; l iterations
define miler_rabin(n, l);
lvars n, l, i;
for i from 1 to l do
  if not (miler_rabin1(n, random (n - 2) + 1)) then
    return(false);
  endif;
endfor;
return(true);
enddefine;

;;; We search for at most 20 numbers
vars a, i, j = 0;
for i from 1 to 1000 do
  2*random(100000000000111111) + 1 -> a;
  if miler_rabin(a, 20) then
    printf(a, '%p\n');
    j + 1 -> j;
    if j = 20 then quitloop endif;
  endif;
endfor;

