/*
  Propositional theorem proving using Hao Wang method
*/

define is_axiom(seq);
   lvars l1 = seq(1), l2 = seq(2);
   lvars var_map = newanyproperty([], 20, 2, 15, false, false, "perm", 
                                 false, false);
   lvars x;
   for x in l1 do
     if isword(x) then
               true -> var_map(x);
     endif;
   endfor;
   for x in l2 do
     if isword(x) and var_map(x) then return(true); endif;
   endfor;
   return(false);
enddefine;

vars wang_flatten_pos;

define wang_flatten_neg(list, neg, pos);
   lvars x;
   for x in list do
     if isword(x) then
        conspair(x, neg) -> neg;
     elseif x(1) = "-" then
        wang_flatten_pos(back(x), neg, pos) -> (neg, pos);
     elseif x(1) = "&" then
        wang_flatten_neg(back(x), neg, pos) -> (neg, pos);
     else
        conspair(x, neg) -> neg;
     endif;
   endfor;
   return(neg, pos);
enddefine;

define wang_flatten_pos(list, neg, pos);
   lvars x;
   for x in list do
     if isword(x) then
        conspair(x, pos) -> pos;
     elseif x(1) = "-" then
        wang_flatten_neg(back(x), neg, pos) -> (neg, pos);
     elseif x(1) = "|" then
        wang_flatten_pos(back(x), neg, pos) -> (neg, pos);
     else
        conspair(x, pos) -> pos;
     endif;
   endfor;
   return(neg, pos);
enddefine;


define hao_wang(seq, pref);
   lvars seq, pref, neg = seq(1), pos = seq(2);
   wang_flatten_neg(neg, [], pos) -> (neg, pos);
   wang_flatten_pos(pos, neg, []) -> (neg, pos);
   printf(pref, '%p');
   printf({^neg ^pos}, '%p\n');
   if is_axiom({^neg ^pos}) then return(true); endif;
   pref <> '   ' -> pref;
   lvars x, x1, ol = neg, nl = [];
   while ol /= [] do
     destpair(ol) -> (x, ol);
     if ispair(x) and x(1) = "|" then
       for x1 in back(x) do
         if not(hao_wang({[^x1 ^^ol ^^nl] ^pos}, pref)) then 
           return(false) 
         endif;
       endfor;
       return(true);
     endif;
     conspair(x, nl) -> nl;
   endwhile;
   pos -> ol;
   while ol /= [] do
     destpair(ol) -> (x, ol);
     if ispair(x) and x(1) = "&" then
       for x1 in back(x) do
         if not(hao_wang({^neg [^x1 ^^ol ^^nl]}, pref)) then 
           return(false)
         endif;
       endfor;
       return(true);
     endif;
     conspair(x, nl) -> nl;
   endwhile;
   return(false);
enddefine;

  hao_wang({[] [[| p [- p]]]}, '') =>
  hao_wang({[] [[| [- [| [- p] q]] [| [- [- q]] [- p]]]]}, '') =>
  hao_wang({[] [[& [| p1 [- p1]] [| p2 [- p2]] [| p3 [- p3]] [| p4 [- p4]]]]}, 
           '') =>
/*
  [| p5 [- p5]] [| p6 [- p6]] [| p7 [- p7]] [| p8 [- p8]]
*/       
